"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var node_crypto_1 = require("node:crypto");
var generateHMACKey_1 = __importDefault(require("./hmac/generateHMACKey"));
var generateAESKey_1 = __importDefault(require("./aes/generateAESKey"));
var algorithms_1 = require("./algorithms");
var optionsSchema = {
    extractable: true,
    toKeyObject: false
};
function isOptionsValid(options) {
    return Object.keys(options).filter(function (key) { return !optionsSchema.hasOwnProperty(key); });
}
function isInputAlgorithmValid(alg) {
    for (var algType in algorithms_1.SYNC_ALGS) {
        if (algorithms_1.SYNC_ALGS[algType].includes(alg)) {
            return true;
        }
    }
    return false;
}
function generateSecret(alg, options) {
    var _a;
    if (options === void 0) { options = optionsSchema; }
    return __awaiter(this, void 0, void 0, function () {
        var secret, extractable, optionErrors;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    extractable = (_a = options.extractable) !== null && _a !== void 0 ? _a : true;
                    optionErrors = isOptionsValid(options);
                    if (optionErrors.length > 0) {
                        optionErrors.forEach(function (key) {
                            throw new Error("".concat(key, " is not a valid option for generateSecret()"));
                        });
                    }
                    if (!isInputAlgorithmValid(alg)) {
                        throw new Error('Invalid Symmetric Algorithm');
                    }
                    if (!algorithms_1.SYNC_ALGS.HMAC_SHA.includes(alg)) return [3 /*break*/, 2];
                    return [4 /*yield*/, (0, generateHMACKey_1.default)(alg, extractable, ['sign', 'verify'])];
                case 1:
                    secret = _b.sent();
                    _b.label = 2;
                case 2:
                    if (!algorithms_1.SYNC_ALGS.AES_KW.includes(alg)) return [3 /*break*/, 4];
                    return [4 /*yield*/, (0, generateAESKey_1.default)(alg, extractable, ['wrapKey', 'unwrapKey'])];
                case 3:
                    secret = _b.sent();
                    _b.label = 4;
                case 4:
                    if (!algorithms_1.SYNC_ALGS.AES_GCM.includes(alg)) return [3 /*break*/, 8];
                    if (!alg.endsWith('KW')) return [3 /*break*/, 6];
                    return [4 /*yield*/, (0, generateAESKey_1.default)(alg, extractable, ['wrapKey', 'unwrapKey'])];
                case 5:
                    secret = _b.sent();
                    return [3 /*break*/, 8];
                case 6: return [4 /*yield*/, (0, generateAESKey_1.default)(alg, extractable, ['encrypt', 'decrypt'])];
                case 7:
                    secret = _b.sent();
                    _b.label = 8;
                case 8: return [2 /*return*/, (options.toKeyObject && secret)
                        ? node_crypto_1.KeyObject.from(secret)
                        : secret];
            }
        });
    });
}
exports.default = generateSecret;
